function [K,U,S,psamp] = OpenCurveCovarianceAndSample(mu,q,X,numsamp)

% Inputs: mu,q,X = outputs from FindMean3Dcurve, X1=original data
% Outputs: U = principal directions of variation
    
[~,T,n] = size(q);

qmean = mu;

% compute inverse exponential map
for i = 1:n
    v = FindGeodesicVector(qmean,q(:,:,i),X(:,:,i),0);
    Dat(i,1:T) = v(1,:);
    Dat(i,T+1:2*T) = v(2,:);
    Dat(i,2*T+1:3*T) = v(3,:);
end

% compute covariance and PCA
K = cov(Dat);
[U,S,~] = svd(K);
mu = mean(Dat);

% display random samples from wrapped Gaussian distribution
figure(17),clf,hold on
idx=1;
while(idx<numsamp+1)
        v = mu + zeros(1,3*T);
        for i=1:(n-1)
            c(i) = randn*sqrt(S(i,i));
            v = v + c(i)*U(:,i)';
        end
        
        qsamp(1,1:T) = v(1:T);
        qsamp(2,1:T) = v(T+1:2*T);
        qsamp(3,1:T) = v(2*T+1:3*T);
        
        nqsamp = sqrt(InnerProd_Q(qsamp,qsamp));
        qsamp = cos(nqsamp)*qmean + sin(nqsamp)*qsamp/nqsamp; 
        psamp(:,:,idx) = q_to_curve(qsamp);
        plot3(psamp(1,:,idx),psamp(2,:,idx)+.25*(idx-1),psamp(3,:,idx),'LineWidth',2)
        idx=idx+1;
end
axis equal off

% This will display the 3 main directions of variation from -1.5SD to
% +1.5SD (mean is in red)
figure(30); clf; hold on;
for j=1:3
    for t=1:5
        c = 3*((t-3)/4)*sqrt(S(j,j));
        v = mu + c*U(:,j)';
        qsamp(1,1:T) = v(1:T);
        qsamp(2,1:T) = v(T+1:2*T);
        qsamp(3,1:T) = v(2*T+1:3*T);
        
        nqsamp = sqrt(InnerProd_Q(qsamp,qsamp));
        qsamp = cos(nqsamp)*qmean + sin(nqsamp)*qsamp/nqsamp; 
        psamp = q_to_curve(qsamp);

        if t == 3
            subplot(3,1,j),z = plot3(psamp(1,:)+(t-1)*.29,psamp(2,:),psamp(3,:),'r','LineWidth',2);
            hold on
            axis equal off;
        else
            subplot(3,1,j),z = plot3(psamp(1,:)+(t-1)*0.29,psamp(2,:),psamp(3,:),'LineWidth',2);
            hold on
            axis equal off;
        end
    end
end
end