function [dist]=my3Dgeod(X1,X2)
 
%input: two curves X1 and X2 as 2xn or 3xn depending on dimensinality
%output: dist=distance

% Load some parameters, no need to change this
lam = 0;

% What displays you want to see, set to 1 if you want to see figures
Disp_geodesic_between_the_curves = 1;
Disp_registration_between_curves = 1;
Disp_optimal_reparameterization = 1;

% Resample the curves to have N points according to arc-length
N = 100;
X1 = ReSampleCurve(X1,N);
X2 = ReSampleCurve(X2,N);

%Center curves, not really needed but good for display purposes
X1 = X1 - repmat(mean(X1')',1,size(X1,2));
X2 = X2 - repmat(mean(X2')',1,size(X2,2));

% Form the SRVFs for representing curves
[q1] = curve_to_q(X1);
[q2] = curve_to_q(X2);

% Find best rotation
A = q1*q2';
[U,S,V] = svd(A);
if det(A)> 0
    Ot = U*V';
else
    Ot = U*([V(:,1) V(:,2) -V(:,3)])';
end
X2 = Ot*X2;
q2 = Ot*q2;

% Solve the registration problem and display the optimal warping function
[gamI] = DynamicProgrammingQ_Adam(q2,q1,lam,0);
gamI = (gamI-gamI(1))/(gamI(end)-gamI(1));
X2n = Group_Action_by_Gamma_Coord(X2,gamI);
q2n = curve_to_q(X2n);
if Disp_optimal_reparameterization
    figure(100); clf;
    plot(linspace(0,1,N),gamI,'LineWidth',2)
    axis square
    title('Optimal Warping','FontSize',24)
    set(gca,'FontSize',20)
end

% Find optimal rotation
A = q1*q2n';
[U,S,V] = svd(A);
if det(A)> 0
    Ot = U*V';
else
    Ot = U*([V(:,1) V(:,2) -V(:,3)])';
end
X2n = Ot*X2n;
q2n = Ot*q2n;

% Computing geodesic distance and forming geodesic deformation between
% shapes
dist = acos(InnerProd_Q(q1,q2n));
sprintf('The distance between the two curves is %0.3f',dist)

if(Disp_geodesic_between_the_curves)
    if (size(X1,1)==2)
        for t=1:7
            s = dist*(t-1)/6;
            PsiQ(:,:,t) = (sin(dist - s)*q1 + sin(s)*q2n)/sin(dist);
            PsiX(:,:,t) = q_to_curve(PsiQ(:,:,t));
        end
        figure(4); clf; axis equal; hold on;
        for t=1:7
            z = plot(0.5*t + PsiX(1,:,t), PsiX(2,:,t),'r-');
            set(z,'LineWidth',[2],'color',[(t-1)/99 (t-1)/198 0]);
        end
        axis off;
    else
        for t=1:7
            s = dist*(t-1)/6;
            PsiQ(:,:,t) = (sin(dist - s)*q1 + sin(s)*q2n)/sin(dist);
            PsiX(:,:,t) = q_to_curve(PsiQ(:,:,t));
        end
        figure(4); clf; axis equal; hold on;
        for t=1:7
            z = plot3(0.5*t + PsiX(1,:,t), PsiX(2,:,t), PsiX(3,:,t),'r-');
            set(z,'LineWidth',[2],'color',[(t-1)/99 (t-1)/198 0]);
        end
        axis off;
    end
end

% Displaying the correspondence
if(Disp_registration_between_curves)
    X2n=PsiX(:,:,end);
    X1=PsiX(:,:,1);
    if (size(X1,1)==2)
        figure(3); clf;
        z = plot3(X1(1,:), X1(2,:), X1(3,:),'r');
        set(z,'LineWidth',[2]);
        axis off;
        hold on;
        z = plot(X2n(1,:), 0.15+X2n(2,:),'b-+');
        set(z,'LineWidth',[3]);
        N = size(X1,2);
        for j=1:N/15
            i = j*15;
            plot([X1(1,i) X2n(1,i)],[X1(2,i) 0.15+X2n(2,i)], 'k');
        end
    else
        figure(3); clf;
        z = plot3(X1(1,:), X1(2,:), X1(3,:),'r');
        set(z,'LineWidth',[2]);
        axis off;
        hold on;
        z = plot3(X2n(1,:), 0.15+X2n(2,:), X2n(3,:),'b-+');
        set(z,'LineWidth',[3]);
        N = size(X1,2);
        for j=1:N/15
            i = j*15;
            plot3([X1(1,i) X2n(1,i)],[X1(2,i) 0.15+X2n(2,i)], [X1(3,i) X2n(3,i)], 'k');
        end
    end
end